@@ -19,7 +19,7 @@ from scipy.optimize import curve_fit
 from scipy.stats import norm
 
 from .. import utils
-from ..utils import plots
+from ..utils import plots, dose_tools
 
 
 
@@ -138,7 +138,7 @@ class ParametricModel(ABC):
         except:
             return None
 
-    def fit(self, d1, d2, E, drug1_model=None, drug2_model=None, use_jacobian = True, p0=None, bootstrap_iterations=0, **kwargs):
+    def fit(self, d1, d2, E, drug1_model=None, drug2_model=None, use_jacobian = True, p0=None, bootstrap_iterations=0, seed=None, **kwargs):
         """Fit the model to data.
 
         Parameters
@@ -166,13 +166,17 @@ class ParametricModel(ABC):
 
         p0 : tuple, default=None
             Initial guess for the parameters. If p0 is None (default), drug1_model and drug2_model will be used to obtain an initial guess. If they are also None, they will be fit to the data. If they fail to fit, the initial guess will be E0=max(E), Emax=min(E), h=1, C=median(d), and all synergy parameters are additive (i.e., at the boundary between antagonistic and synergistic)
+
+        seed : int, default=None
+            If not None, used as numpy.random.seed(start_seed) at the beginning of bootstrap resampling
         
         kwargs
             kwargs to pass to scipy.optimize.curve_fit()
         """
 
-        d1 = np.asarray(d1)
-        d2 = np.asarray(d2)
+        if seed is not None: np.random.seed(seed)
+        d1 = np.asarray(d1, dtype=np.float64)
+        d2 = np.asarray(d2, dtype=np.float64)
 
         E = np.asarray(E)
 
@@ -261,7 +265,7 @@ class ParametricModel(ABC):
         pass
 
     
-    def _bootstrap_resample(self, d1, d2, E, use_jacobian, bootstrap_iterations, **kwargs):
+    def _bootstrap_resample(self, d1, d2, E, use_jacobian, bootstrap_iterations, seed=None, **kwargs):
         """Internal function to identify confidence intervals for parameters
         """
 
@@ -278,7 +282,9 @@ class ParametricModel(ABC):
 
         xdata = np.vstack((d1,d2))
 
+        #if start_seed is not None: np.random.seed(start_seed)
         for iteration in range(bootstrap_iterations):
+            #if start_seed is not None: np.random.seed(start_seed + iteration)
             residuals_step = norm.rvs(loc=0, scale=sigma_residuals, size=n_data_points)
 
             # Add random noise to model prediction
@@ -337,8 +343,8 @@ class ParametricModel(ABC):
         E = self.E(d1, d2)
         plots.plot_heatmap(d1, d2, E, cmap=cmap, **kwargs)
 
-    def plot_residual_heatmap(self, d1, d2, E, cmap="RdBu", **kwargs):
-        """Plots the model's effect, E(d1, d2) as a heatmap
+    def plot_residual_heatmap(self, d1, d2, E, cmap="RdBu", center_on_zero=True, **kwargs):
+        """Plots the residuals of the fit model as a heatmap
 
         Parameters
         ----------
@@ -352,7 +358,7 @@ class ParametricModel(ABC):
             Observed drug effects
 
         cmap : string, default="RdBu"
-            Colorscale for the plot
+            Colormap for the plot
         
         kwargs
             kwargs passed to synergy.utils.plots.plot_heatmap()
@@ -362,7 +368,41 @@ class ParametricModel(ABC):
             return
         
         Emodel = self.E(d1, d2)
-        plots.plot_heatmap(d1, d2, E-Emodel, cmap=cmap, **kwargs)
+        plots.plot_heatmap(d1, d2, E-Emodel, cmap=cmap, center_on_zero=center_on_zero, **kwargs)
+
+    @abstractmethod
+    def _reference_E(self, d1, d2):
+        pass
+
+    def plot_reference_heatmap(self, d1, d2, cmap="YlGnBu", **kwargs):
+        if not self._is_parameterized():
+            #raise ModelNotParameterizedError()
+            return
+
+        Ereference = self._reference_E(d1, d2)
+        plots.plot_heatmap(d1, d2, Ereference, cmap=cmap, **kwargs)
+
+    def plot_reference_surface(self, d1, d2, cmap="YlGnBu", **kwargs):
+        if not self._is_parameterized():
+            return
+        Ereference = self._reference_E(d1, d2)
+        plots.plot_surface_plotly(d1, d2, Ereference, cmap=cmap, **kwargs)
+
+    def plot_delta_heatmap(self, d1, d2, cmap="PRGn", center_on_zero=True, **kwargs):
+        if not self._is_parameterized():
+            #raise ModelNotParameterizedError()
+            return
+        Ereference = self._reference_E(d1, d2)
+        Emodel = self.E(d1, d2)
+        plots.plot_heatmap(d1, d2, Ereference-Emodel, cmap=cmap, center_on_zero=center_on_zero, **kwargs)
+
+    def plot_delta_surface(self, d1, d2, cmap="PRGn", center_on_zero=True, **kwargs):
+        if not self._is_parameterized():
+            return
+        Ereference = self._reference_E(d1, d2)
+        Emodel = self.E(d1, d2)
+        plots.plot_surface_plotly(d1, d2, Ereference-Emodel, cmap=cmap, center_on_zero=center_on_zero, **kwargs)
+    
 
     def plot_surface_plotly(self, d1, d2, cmap="YlGnBu", **kwargs):
         """Plots the model's effect, E(d1, d2) as a surface using synergy.utils.plots.plot_surface_plotly()
@@ -385,5 +425,7 @@ class ParametricModel(ABC):
             #raise ModelNotParameterizedError()
             return
         
+        # d1 and d2 may come from data, and have replicates. This would cause problems with surface plots (replicates in scatter_points are fine, but replicates in the surface itself are not)
+        d1, d2 = dose_tools.remove_replicates(d1, d2)
         E = self.E(d1, d2)
         plots.plot_surface_plotly(d1, d2, E, cmap=cmap, **kwargs)
\ No newline at end of file
