@@ -35,6 +35,10 @@ try:
 except ImportError as e:
     pass
 
+def get_extension(fname):
+    if not "." in fname: return ""
+    return fname.split(".")[-1]
+
 def plot_heatmap(d1, d2, E, ax=None, fname=None, title="", xlabel="Drug 1", ylabel="Drug 2", figsize=None, cmap="PRGn", aspect='equal', vmin=None, vmax=None, center_on_zero=False, logscale=True, nancolor="#BBBBBB", **kwargs):
         if (not matplotlib_import):
             raise ImportError("matplotlib must be installed to plot")
@@ -202,15 +206,17 @@ def interp(x, x0, x1, y0, y1):
 def plot_surface_plotly(d1, d2, E, scatter_points=None,       \
                  elev=20, azim=19, fname="plot.html", zlim=None, cmap='viridis', logscale=True,         \
                  xlabel="Drug 1", ylabel="Drug 2", zlabel="z", \
-                 vmin=None, vmax=None, auto_open=True, opacity=0.8, \
-                 center_on_zero=False):
+                 vmin=None, vmax=None, auto_open=False, opacity=0.8, \
+                 center_on_zero=False, figsize=(1000,800), \
+                 fontsize=18):
     if (not plotly_import):
         raise ImportError("plot_surface_plotly() requires plotly to be installed.")
     
-    d1 = np.array(d1, copy=True)
-    d2 = np.array(d2, copy=True)
+    d1 = np.array(d1, copy=True, dtype=np.float64)
+    d2 = np.array(d2, copy=True, dtype=np.float64)
     E = np.asarray(E)
 
+    extension = get_extension(fname)
 
     if logscale:
         d1 = utils.remove_zeros(d1)
@@ -265,8 +271,8 @@ def plot_surface_plotly(d1, d2, E, scatter_points=None,       \
     if scatter_points is not None:
         #d1scatter = utils.remove_zeros(np.asarray(scatter_points['drug1.conc']))
         #d2scatter = utils.remove_zeros(np.asarray(scatter_points['drug2.conc']))
-        d1scatter = np.array(scatter_points['drug1.conc'], copy=True)
-        d2scatter = np.array(scatter_points['drug2.conc'], copy=True)
+        d1scatter = np.array(scatter_points['drug1.conc'], copy=True, dtype=np.float64)
+        d2scatter = np.array(scatter_points['drug2.conc'], copy=True, dtype=np.float64)
         if logscale:
             zero_mask_1 = np.where(d1scatter <= 0)
             pos_mask_1 = np.where(d1scatter > 0)
@@ -314,8 +320,8 @@ def plot_surface_plotly(d1, d2, E, scatter_points=None,       \
             y=0.88, 
             z=0.64
         ), 
-        width=1000, 
-        height=800, 
+        width=figsize[0], 
+        height=figsize[1], 
         margin=dict(
             l=100, 
             r=100, 
@@ -329,7 +335,7 @@ def plot_surface_plotly(d1, d2, E, scatter_points=None,       \
             aspectmode="cube"
         ),
         font=dict(
-            size=18
+            size=fontsize
         )
     )
 
@@ -338,16 +344,24 @@ def plot_surface_plotly(d1, d2, E, scatter_points=None,       \
 
 
     if fname is not None:
-        offline.plot(fig, filename=fname, auto_open=auto_open)
+        if extension=="html":
+            offline.plot(fig, filename=fname, auto_open=auto_open)
+        else:
+            if not extension.lower() in ['png','jpeg','jpg','webp','svg','pdf','eps']:
+                extension='png'
+            from plotly.io import write_image
+            write_image(fig, fname, format=extension)
     else:
         fig.show()
 
 
 def plotly_isosurfaces(d1, d2, d3, E, fname=None,     \
-            cmap='viridis', xlabel="Drug 1", ylabel="Drug 2", zlabel="Drug 3",     \
-            vmin=None, vmax=None, auto_open=True, opacity=0.6,      \
+            cmap='viridis', xlabel="Drug 1", ylabel="Drug 2", \
+            zlabel="Drug 3",     \
+            vmin=None, vmax=None, auto_open=False, opacity=0.6,      \
             logscale=True, isomin=None, isomax=None,                \
-            center_on_zero=False, surface_count=10):
+            center_on_zero=False, surface_count=10, \
+            figsize=(1000,800), fontsize=18):
     
     d1 = np.asarray(d1)
     d2 = np.asarray(d2)
@@ -419,8 +433,8 @@ def plotly_isosurfaces(d1, d2, d3, E, fname=None,     \
             y=0.88, 
             z=0.64
         ), 
-        width=1000, 
-        height=800, 
+        width=figsize[0],
+        height=figsize[1],
         margin=dict(
             l=100, 
             r=100, 
@@ -434,7 +448,7 @@ def plotly_isosurfaces(d1, d2, d3, E, fname=None,     \
             aspectmode="cube"
         ),
         font=dict(
-            size=18
+            size=fontsize
         )
     )
 
